/*
 * ALSA Patch Bay
 *
 * Copyright (C) 2002 Robert Ham (node@users.sourceforge.net)
 *
 * You have permission to use this file under the GNU General
 * Public License, version 2 or later.  See the file COPYING
 * for the full text.
 *
 */

#ifndef __APB_ALSA_H__
#define __APB_ALSA_H__

#include <list>
#include <string>
#include <iostream>
#include <alsa/asoundlib.h>

#include "apb.h"

namespace APB {
namespace Alsa {

class Driver;

class Addr : public APB::Addr {
  private:
    snd_seq_addr_t _addr;
    Driver * _driver;
  public:
    Addr (const snd_seq_addr_t& addr, Driver * driver)
     : _addr (addr), _driver (driver) {}
    Addr (const Addr& addr)
     : _addr (addr._addr), _driver (addr._driver) {}
    const snd_seq_addr_t& addr () const { return _addr; }
//    void addr (const snd_seq_addr_t& addr) { _addr = addr; }
    int client () const { return _addr.client; }
    int port () const { return _addr.port; }
    virtual bool equals (const APB::Addr * addr) const {
      if (!addr) {
        std::cerr << DEBUG_STRING << "addr is null; returning false" << std::endl;
        return false;
      }
      return ((Alsa::Addr *)addr)->_addr.client == _addr.client
             && ((Alsa::Addr *)addr)->_addr.port == _addr.port;
    }
    virtual APB::Driver * driver () const { return (APB::Driver *) _driver; }
    virtual std::string getName () const;
};

class Driver : public APB::Driver {
  private:
    snd_seq_t * _seq;
    std::list<APB::Addr *> _readPorts;
    std::list<APB::Addr *> _writePorts;
    std::list<APB::Subscription> _subscriptions;
    
    std::string _title;
    bool _jfd;
    APB::UI * _ui;
    
    void refreshPorts (std::list<APB::Addr *>&, unsigned int);
    void doPortSubscription (snd_seq_port_subscribe_t *,
                             const APB::Addr *,
                             const APB::Addr *);
    void doPortUnsubscription (snd_seq_port_subscribe_t *,
                             const APB::Addr *,
                             const APB::Addr *);
    void subscribePorts (const APB::Addr *,
                         const APB::Addr *,
                         bool);
  public:
    Driver (std::string, int, char **);
    virtual ~Driver () {
    }
    virtual std::string findClientName (const APB::Addr *) const;
    virtual std::string findPortName (const APB::Addr *) const;
    virtual const std::list<APB::Addr *>& getReadPorts ();
    virtual const std::list<APB::Addr *>& getWritePorts ();
    virtual const std::list<APB::Subscription>& getSubscriptions ();
    virtual void refreshPorts ();
    virtual void refreshSubscriptions ();
    virtual void subscribePorts (const APB::Addr *, const APB::Addr *);
    virtual void unsubscribePorts (const APB::Addr *, const APB::Addr *);
    virtual void setJFD (bool);
    virtual void setUI (APB::UI * ui);
};

} /* namespace Alsa */
} /* namespace APB */

#endif /* __APB_ALSA_H__ */
